package de.firstdata.ipg.api.client.action.recurringpayment.object;

import java.net.URL;
import java.util.Date;

import de.firstdata.ipg.api.client.action.recurringpayment.InstallRecurringPayment;
import de.firstdata.ipg.api.client.transaction.Amount;
import de.firstdata.ipg.api.client.transaction.creditcard.CreditCard;
import de.firstdata.ipg.api.client.transaction.dedirectdebit.DEDirectDebit;
import de.firstdata.ipg.api.schema.AdditionalRequestParameters;
import de.firstdata.ipg.api.schema.CardEnrolledType;
import de.firstdata.ipg.api.schema.CreditCard3DSecure;
import de.firstdata.ipg.api.schema.Function;
import de.firstdata.ipg.api.schema.RecurringPayment;
import de.firstdata.ipg.api.schema.RecurringPaymentInformation;
import de.firstdata.ipg.api.schema.Secure3DAuthenticationRequest;
import de.firstdata.ipg.api.schema.Secure3DRequest;
import de.firstdata.ipg.api.schema.Secure3DVerificationRequest;
import de.firstdata.ipg.api.schema.TransactionDetails;
import de.firstdata.ipg.api.schema.Upop;
import de.firstdata.ipg.api.schema.Validate;

public class InstallRecurringPaymentObject extends RecurringPaymentObject implements InstallRecurringPayment {

	private InstallRecurringPaymentObject(final Amount amount, final int installmentFrequency, final String installmentPeriod, final int maximumFailures,
			final String recurringStartDate) {
		super(Function.INSTALL);
		setPayment(amount);
		final RecurringPaymentInformation recurringPaymentInformation = getRecurringPaymentInformation();
		recurringPaymentInformation.setInstallmentFrequency(Integer.valueOf(installmentFrequency));
		recurringPaymentInformation.setInstallmentPeriod(installmentPeriod);
		recurringPaymentInformation.setMaximumFailures(Integer.valueOf(maximumFailures));
		recurringPaymentInformation.setRecurringStartDate(recurringStartDate);
	}

	public InstallRecurringPaymentObject(final Amount amount, final CreditCard creditCard, final int installmentFrequency, final String installmentPeriod,
			final int maximumFailures, final String recurringStartDate) {
		this(amount, installmentFrequency, installmentPeriod, maximumFailures, recurringStartDate);
		setCreditCard(creditCard);
	}

	public InstallRecurringPaymentObject(final Amount amount, final CreditCard creditCard, final int installmentFrequency, final String installmentPeriod,
			final int maximumFailures, final Date recurringStartDate) {
		this(amount, creditCard, installmentFrequency, installmentPeriod, maximumFailures, getFormattedDate(recurringStartDate));
	}

	public InstallRecurringPaymentObject(final Amount amount, final DEDirectDebit deDirectDebit, final int installmentFrequency, final String installmentPeriod,
			final int maximumFailures, final String recurringStartDate) {
		this(amount, installmentFrequency, installmentPeriod, maximumFailures, recurringStartDate);
		setDEDirectDebit(deDirectDebit);
	}

	public InstallRecurringPaymentObject(final Amount amount, final DEDirectDebit deDirectDebit, final int installmentFrequency, final String installmentPeriod,
			final int maximumFailures, final Date recurringStartDate) {
		this(amount, deDirectDebit, installmentFrequency, installmentPeriod, maximumFailures, getFormattedDate(recurringStartDate));
	}

	public InstallRecurringPaymentObject(final Amount amount, final String referencedOrderID, final int installmentFrequency, final String installmentPeriod,
			final int maximumFailures, final String recurringStartDate) {
		this(amount, installmentFrequency, installmentPeriod, maximumFailures, recurringStartDate);
		setReferencedOrderID(referencedOrderID);
	}

	public InstallRecurringPaymentObject(final Amount amount, final String referencedOrderID, final int installmentFrequency, final String installmentPeriod,
			final int maximumFailures, final Date recurringStartDate) {
		this(amount, referencedOrderID, installmentFrequency, installmentPeriod, maximumFailures, getFormattedDate(recurringStartDate));
	}

	public InstallRecurringPaymentObject(final Amount amount, final String hostedDataID, final String hostedDataStoreID, final int installmentFrequency,
			final String installmentPeriod, final int maximumFailures, final String recurringStartDate) {
		this(amount, installmentFrequency, installmentPeriod, maximumFailures, recurringStartDate);
		setHostedDataID(hostedDataID);
		setHostedDataStoreID(hostedDataStoreID);
	}

	public InstallRecurringPaymentObject(final Amount amount, final String hostedDataID, final String hostedDataStoreID, final int installmentFrequency,
			final String installmentPeriod, final int maximumFailures, final Date recurringStartDate) {
		this(amount, hostedDataID, hostedDataStoreID, installmentFrequency, installmentPeriod, maximumFailures, getFormattedDate(recurringStartDate));
	}

	@Override
	public void setTransactionOrigin(final String value) {
		getRecurringPayment().setTransactionOrigin(value);
	}

	@Override
	public void setIp(final String ip) {
		getRecurringPayment().setIp(ip);
	}

	@Override
	public void setInvoiceNumber(final String value) {
		getRecurringPayment().setInvoiceNumber(value);
	}

	@Override
	public void setDynamicMerchantName(final String dynamicMerchantName) {
		getRecurringPayment().setDynamicMerchantName(dynamicMerchantName);
	}

	@Override
	public void setPONumber(final String value) {
		getRecurringPayment().setPONumber(value);
	}

	@Override
	public void setAuthenticationValue(final String value) {
		getCreditCard3DSecure().setAuthenticationValue(value);
	}

	@Override
	public void setAuthenticate3dSecureTransaction(final Boolean value) {
		getCreditCard3DSecure().setAuthenticateTransaction(value);
	}

	@Override
	public void setPayerAuthenticationResponse(final String value) {
		getCreditCard3DSecure().setPayerAuthenticationResponse(value);
	}

	@Override
	public void setVerificationResponse(final String value) {
		getCreditCard3DSecure().setVerificationResponse(value == null ? null : CardEnrolledType.valueOf(value));
	}

	@Override
	public void setXID(final String value) {
		getCreditCard3DSecure().setXID(value);
	}

	@Override
	public void setDSRPECI(final String value) {
		getCreditCard3DSecure().setDSRPECI(value);
	}

	@Override
	public void setOverride3DSCountryExclusion(final boolean override3DSCountryExclusion) {
		getCreditCard3DSecure().setOverride3DsCountryExclusion(Boolean.valueOf(override3DSCountryExclusion));
	}

	@Override
	public void setSecure3DMethodNotificationStatus(final String secure3DMethodNotificationStatus) {
		getCreditCard3DSecure().setSecure3DMethodNotificationStatus(secure3DMethodNotificationStatus);
	}

	@Override
	public void setTermUrl(final URL termUrl) {
		getCreditCard3DSecure().setTermUrl(termUrl == null ? null : termUrl.toString());
	}

	@Override
	public void setThreeDSMethodNotificationURL(final URL threeDSMethodNotificationURL) {
		getCreditCard3DSecure().setThreeDSMethodNotificationURL(threeDSMethodNotificationURL == null ? null : threeDSMethodNotificationURL.toString());
	}

	@Override
	public void setThreeDSRequestorChallengeIndicator(final String value) {
		getCreditCard3DSecure().setThreeDSRequestorChallengeIndicator(value);
	}

	@Override
	public void setThreeDSTransType(final String value) {
		getCreditCard3DSecure().setThreeDSTransType(value);
	}

	@Override
	public void setSecure3D2AuthenticationResponse(final String authenticationResponse) {
		getCreditCard3DSecure().setSecure3D2AuthenticationResponse(authenticationResponse);
	}

	@Override
	public void setSecure3D2TransactionStatus(final String transactionStatus) {
		getCreditCard3DSecure().setSecure3D2TransactionStatus(transactionStatus);
	}

	@Override
	public void setSecure3DProtocolVersion(final String protocolVersion) {
		getCreditCard3DSecure().setSecure3DProtocolVersion(protocolVersion);
	}

	@Override
	public void setAcsTransactionId(final String acsTransactionId) {
		getCreditCard3DSecure().setAcsTransactionId(acsTransactionId);
	}

	@Override
	public void setThreeDSServerTransactionId(final String threeDSServerTransactionId) {
		getCreditCard3DSecure().setThreeDSServerTransactionId(threeDSServerTransactionId);
	}

	@Override
	public void setDirectoryServerTransactionId(final String directoryServerTransactionId) {
		getCreditCard3DSecure().setDirectoryServerTransactionId(directoryServerTransactionId);
	}

	private CreditCard3DSecure getCreditCard3DSecure() {
		final RecurringPayment recurringPayment = getRecurringPayment();
		CreditCard3DSecure creditCard3DSecure = recurringPayment.getCreditCard3DSecure();
		if (creditCard3DSecure == null) {
			creditCard3DSecure = new CreditCard3DSecure();
			recurringPayment.setCreditCard3DSecure(creditCard3DSecure);
		}
		return creditCard3DSecure;
	}

	@Override
	public void setAdditionalRequestParameters(final AdditionalRequestParameters additionalRequestParameterss) {
		if (getValidate() != null && getValidate().getTransactionDetails() != null) {
			getValidate().getTransactionDetails().setAdditionalRequestParameters(additionalRequestParameterss);
		}
	}

	@Override
	public void setSecure3DVerificationRequest(final Secure3DVerificationRequest secure3dVerificationRequest) {
		getSecure3DRequest().setSecure3DVerificationRequest(secure3dVerificationRequest);
	}

	@Override
	public void setSecure3DAuthenticationRequest(final Secure3DAuthenticationRequest secure3dAuthenticationRequest) {
		getSecure3DRequest().setSecure3DAuthenticationRequest(secure3dAuthenticationRequest);
	}

	private Secure3DRequest getSecure3DRequest() {
		Secure3DRequest secure3DRequest = getCreditCard3DSecure().getSecure3DRequest();
		if (secure3DRequest == null) {
			secure3DRequest = new Secure3DRequest();
			getCreditCard3DSecure().setSecure3DRequest(secure3DRequest);
		}
		return secure3DRequest;
	}

	private Upop getUpopSecurePlus() {
		final RecurringPayment recurringPayment = getRecurringPayment();
		Upop upop = recurringPayment.getUpop();
		if (upop == null) {
			upop = new Upop();
			recurringPayment.setUpop(upop);
		}
		return upop;
	}

	@Override
	public void setVCode(final String value) {
		getUpopSecurePlus().setVCode(value);
	}

	@Override
	public void setActivateStatus(final String value) {
		getUpopSecurePlus().setActivateStatus(value);
	}

	@Override
	public void setResponseCode(final String value) {
		getUpopSecurePlus().setSendSmsResponseCode(value);
	}

	@Override
	public void setMerchantAdviceCodeSupported(final Boolean merchantAdviceCodeSupported) {
		final Validate validate = getValidate();
		if (validate != null) {
			final TransactionDetails transactionDetails = validate.getTransactionDetails();
			if (transactionDetails != null) {
				transactionDetails.setMerchantAdviceCodeSupported(merchantAdviceCodeSupported);
			}
		}
	}
}
